@@ -18,24 +18,23 @@ NULL
 #' Supports coordinates for spatially-resolved molecule (FISH) data.
 #' Compatible with \code{\link{SpatialImage}}
 #'
-#' @slot molecules (\code{\link[base]{list}}) A named list of
+#' @slot molecules A named list of
 #' \code{\link[SeuratObject:Molecules-class]{Molecules}} objects defining
 #' spatially-resolved molecular coordinates
-#' @slot boundaries (\code{[named]\link[base]{list}}
-#' \{\code{\link[SeuratObject:Segmentation-class]{Segmentation}},
-#' \code{\link[SeuratObject:Centroids-class]{Centroids}}\}) A named list of
+#' @slot boundaries  A named list of
 #' \code{\link[SeuratObject:Segmentation-class]{Segmentation}} and
 #' \code{\link[SeuratObject:Centroids-class]{Centroids}} objects defining
 #' spatially-resolved boundaries
-#' @slot assay (\code{\link[base:character]{character [1L]}}) A character
-#' naming the associated assay of the spatial coordinates
-#' @slot key (\code{\link[base:character]{character [1L]}}) The key
-#' for the spatial coordinates
+#' @slot assay A character naming the associated assay
+#' of the spatial coordinates
+#' @template slot-key
 #'
 #' @exportClass FOV
 #'
 #' @aliases FOV
 #'
+#' @concept fov
+#'
 #' @seealso \code{\link{FOV-methods}}
 #'
 setClass(
@@ -81,6 +80,8 @@ setClass(
 #' @name FOV-methods
 #' @rdname FOV-methods
 #'
+#' @concept fov
+#'
 #' @seealso \code{\link{FOV-class}}
 #'
 NULL
@@ -120,6 +121,7 @@ CreateFOV.Centroids <- function(
   molecules = NULL,
   assay = 'Spatial',
   key = NULL,
+  misc = NULL,
   name = NULL,
   ...
 ) {
@@ -130,7 +132,8 @@ CreateFOV.Centroids <- function(
     coords = coords,
     molecules = molecules,
     assay = assay,
-    key = key
+    key = key,
+    misc = misc %||% list()
   ))
 }
 
@@ -146,6 +149,7 @@ CreateFOV.Centroids <- function(
 #' \code{\link[SeuratObject:Centroids-class]{Centroids}}, or
 #' \code{\link[SeuratObject:Segmentation-class]{Segmentation}}, name
 #' to store coordinates as
+#' @param misc A list of miscellaneous information to store with the object
 #'
 #' @rdname CreateFOV
 #' @method CreateFOV data.frame
@@ -160,6 +164,7 @@ CreateFOV.data.frame <- function(
   molecules = NULL,
   assay = 'Spatial',
   key = NULL,
+  misc = NULL,
   name = NULL,
   ...
 ) {
@@ -179,7 +184,8 @@ CreateFOV.data.frame <- function(
     coords = coords,
     molecules = molecules,
     assay = assay,
-    key = key
+    key = key,
+    misc = misc %||% list()
   ))
 }
 
@@ -193,6 +199,7 @@ CreateFOV.list <- function(
   molecules = NULL,
   assay = 'Spatial',
   key = NULL,
+  misc = NULL,
   ...
 ) {
   # Create a list of Molecules objects if provided; otherwise use an empty list
@@ -206,7 +213,8 @@ CreateFOV.list <- function(
     boundaries = coords,
     molecules = molecules,
     assay = assay,
-    key = key %||% Key(object = assay, quiet = TRUE)
+    key = key %||% Key(object = assay, quiet = TRUE),
+    misc = misc %||% list()
   )
   return(obj)
 }
@@ -320,6 +328,8 @@ FetchData.FOV <- function(
   vars.orig <- vars
   if (is.numeric(x = cells)) {
     cells <- Cells(x = object)[cells]
+  } else if (is.null(cells)) {
+    cells <- Cells(x = object)
   }
   # Find keyed molecules
   object.keys <- Keys(object = object)
@@ -574,6 +584,8 @@ RenameCells.FOV <- function(object, new.names = NULL, ...) {
 #' @name aggregate
 #' @rdname aggregate
 #'
+#' @keywords internal
+#'
 #' @method aggregate FOV
 #' @export
 #'
@@ -899,13 +911,8 @@ setMethod(
 setMethod(
   f = 'initialize',
   signature = 'FOV',
-  definition = function(.Object, molecules, boundaries, assay, key, ...) {
+  definition = function(.Object, ...) {
     .Object <- callNextMethod(.Object, ...)
-    slot(object = .Object, name = 'molecules') <- molecules
-    slot(object = .Object, name = 'boundaries') <- boundaries
-    slot(object = .Object, name = 'assay') <- assay
-    slot(object = .Object, name = 'key') <- key
-    # Reorder cells in boundaries
     .Object <- .OrderCells(object = .Object)
     validObject(object = .Object)
     return(.Object)
@@ -995,9 +1002,33 @@ setMethod(
   }
 )
 
+#' FOV Validity
+#'
+#' @templateVar cls FOV
+#' @template desc-validity
+#'
+#' @section Boundary Validation:
+#' blah
+#'
+#' @section Molecule Validation:
+#' blah
+#'
+#' @name FOV-validity
+#'
+#' @family fov
+#'
+#' @seealso \code{\link[methods]{validObject}}
+#'
 setValidity(
   Class = 'FOV',
   method = function(object) {
+    if (isFALSE(x = getOption(x = "Seurat.object.validate", default = TRUE))) {
+      warn(
+        message = paste("Not validating", class(x = object)[1L], "objects"),
+        class = 'validationWarning'
+      )
+      return(TRUE)
+    }
     valid <- NULL
     # Check boundaries
     nlist <- IsNamedList(
@@ -1029,12 +1060,6 @@ setValidity(
                 "All segmentation boundaries must have cells"
               )
               break
-            } else if (is.unsorted(x = matched.cells)) {
-              valid <- c(
-                valid,
-                "All segmentation boundaries must be ordered"
-              )
-              break
             }
           } else {
             valid <- c(
