@@ -10,7 +10,8 @@
 #' enquo eval_tidy have_name inform is_bare_character is_bare_integerish
 #' is_bare_list is_bare_numeric is_missing is_na is_named is_quosure
 #' missing_arg warn
-#' @importClassesFrom Matrix dgCMatrix
+#' @importClassesFrom Matrix dgCMatrix CsparseMatrix dsparseMatrix generalMatrix
+#' dMatrix sparseMatrix compMatrix Matrix
 #' @useDynLib SeuratObject
 #'
 NULL
@@ -114,6 +115,13 @@ generics::intersect
 # #'
 # Matrix::colMeans
 
+#' @importFrom Matrix t
+#' @export
+#'
+#' @noRd
+#'
+Matrix::t
+
 #' @importFrom progressr handlers
 #' @export
 #'
@@ -262,9 +270,12 @@ setOldClass(Classes = 'package_version')
   )
   future <- isTRUE(x = getOption(x = opt, default = FALSE)) ||
     packageVersion(pkg = 'SeuratObject', lib.loc = lib.loc) >= version
-  if (requireNamespace('Seurat', quietly = TRUE)) {
+  # If `Seurat` is installed, check against it's version.
+  # This is a (probably bad/hacky) workaround to avoid calling `requireNamespace`
+  # and forcing us to list `Seurat` as a dependency.
+  if (nzchar(system.file(package = "Seurat"))) {
     future <- future ||
-      packageVersion(pkg = 'Seurat', lib.loc = lib.loc) >= version
+      packageVersion("Seurat", lib.loc = lib.loc) >= version
   }
   return(future)
 }
